package com.perforce.maven.scm.provider.p4.util;

import java.io.File;

import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

public class P4UrlParserTest
{

    @BeforeClass
    public static void beforeClass()
    {
        // we dont want external p4maven-settings.xml influence this test
        P4Utils.setP4MavenSettingsDirecotry( new File( "some-unknown-location" ) );
    }

    @Test
    public void testUrlWithEnvHost()
    {

        String p4PortEnv = System.getenv( P4ScmUrl.P4PORT_KEY );
        if ( p4PortEnv == null )
        {
            return;
        }

        P4ScmUrl p4ScmUrl = P4ScmUrl.parse( "scm:p4://depot/project" );

        Assert.assertEquals( "//depot/project", p4ScmUrl.getPath() );
        Assert.assertNotNull( p4ScmUrl.getHost() );

    }

    @Test
    public void testUrlWithSystemPropertyHost()
    {
        // this prevents concurrent junit test
        String currentP4PortProp = System.getProperty( P4ScmUrl.P4PORT_KEY );
        if ( currentP4PortProp == null )
        {
            currentP4PortProp = "";
        }

        try
        {
            System.setProperty( P4ScmUrl.P4PORT_KEY, "host:3111" );

            P4ScmUrl p4ScmUrl = P4ScmUrl.parse( "scm:p4:ssl:host://depot/project" );

            Assert.assertEquals( "//depot/project", p4ScmUrl.getPath() );
            Assert.assertEquals( "host", p4ScmUrl.getHost() );
            Assert.assertNull( p4ScmUrl.getProtocol() );
            Assert.assertEquals( 3111, p4ScmUrl.getPort() );
        }
        finally
        {
            System.setProperty( P4ScmUrl.P4PORT_KEY, currentP4PortProp );
        }
    }

    @Test( expected = IllegalArgumentException.class )
    public void testBadProviderWithHostInfo()
    {
        P4ScmUrl.parse( "scm:p5:ssl:host://depot/project" );
    }

    @Test( expected = IllegalArgumentException.class )
    public void testBadProviderWithOutHostInfo()
    {
        P4ScmUrl.parse( "scm:p5://depot/project" );
    }

    @Test( expected = IllegalArgumentException.class )
    public void testBadProtocol()
    {
        P4ScmUrl.parse( "scm:p4:xxx:host://depot/project" );
    }

    @Test
    public void testFullUrl()
    {
        String url = "scm:p4:ssl:host:1234://depot/project";

        P4ScmUrl p4ScmUrl = P4ScmUrl.parse( url );
        Assert.assertEquals( "//depot/project", p4ScmUrl.getPath() );
        Assert.assertEquals( "host", p4ScmUrl.getHost() );
        Assert.assertEquals( "ssl", p4ScmUrl.getProtocol() );
        Assert.assertEquals( 1234, p4ScmUrl.getPort() );
        Assert.assertEquals( url, p4ScmUrl.getUrl() );

        url = "scm:p4:tcp:host://depot/project";
        p4ScmUrl = P4ScmUrl.parse( url );
        Assert.assertEquals( "//depot/project", p4ScmUrl.getPath() );
        Assert.assertEquals( "host", p4ScmUrl.getHost() );
        Assert.assertEquals( "tcp", p4ScmUrl.getProtocol() );
        Assert.assertEquals( 0, p4ScmUrl.getPort() );
        Assert.assertEquals( url, p4ScmUrl.getUrl() );
    }

    @Test
    public void testUrlWithP4MavenSettings()
    {
        try
        {
            P4Utils.resetSettings();
            P4Utils.setP4MavenSettingsDirecotry( new File( "src/test/data" ) );
            String url = "scm:p4://depot/project";

            P4ScmUrl p4ScmUrl = P4ScmUrl.parse( url );
            Assert.assertEquals( "//depot/project", p4ScmUrl.getPath() );
            Assert.assertEquals( "localhost", p4ScmUrl.getHost() );
            Assert.assertNull( p4ScmUrl.getProtocol() );
            Assert.assertEquals( 666, p4ScmUrl.getPort() );
            Assert.assertEquals( "scm:p4:localhost:666://depot/project", p4ScmUrl.getUrl() );

        }
        finally
        {
            P4Utils.setP4MavenSettingsDirecotry( new File( "some-unknown-location" ) );
        }
    }

}
